/*
 * Copyright 2006-2007 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.openspaces.maven.plugin;

import java.io.File;
import java.io.IOException;

import com.gigaspaces.start.XapModules;

import org.apache.maven.artifact.repository.ArtifactRepository;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.springframework.util.FileCopyUtils;

/**
 * Installs the GigaSpaces license file in the local repository.
 */
@SuppressWarnings({"JavaDoc", "unused"})
@Mojo(name = "install-license", threadSafe = true, requiresProject = false)
public class InstallLicenseMojo extends AbstractOpenSpacesMojo {


    @Parameter(property = "file", required = true)
    private File file;


    @Parameter(property = "localRepository", required = true, readonly = true)
    @SuppressWarnings("WeakerAccess")
    protected ArtifactRepository localRepository;


    @Parameter(property = "version", required = true)
    protected String version;


    public void executeMojo() throws MojoExecutionException {
        if (!file.exists()) {
            throw new MojoExecutionException("GigaSpaces license file not found at " + file.getAbsolutePath());
        }

        if (version == null || version.length() == 0) {
            throw new MojoExecutionException("GigaSpaces version is not specified");
        }

        File baseDir = new File(localRepository.getBasedir());
        if (!baseDir.exists()) {
            throw new MojoExecutionException("The local repository was not found at " + baseDir.getAbsolutePath());
        }
        File targetDir = new File(localRepository.getBasedir() + "/com/gigaspaces/" + XapModules.DATA_GRID.getArtifactName() + "/" + version + "/");
        if (!targetDir.exists()) {
            throw new MojoExecutionException("The " + XapModules.CORE_COMMON.getJarFileName() + " artifact is not installed at [" + targetDir.getAbsolutePath() + "]");
        }

        try {
            File target = new File(targetDir, "xap-license.txt");
            PluginLog.getLog().info("Copying license file to: " + target.getAbsolutePath());
            FileCopyUtils.copy(file, target);
        } catch (IOException e) {
            throw new MojoExecutionException("Failed to copy GigaSpaces license file.", e);
        }
    }
}
